## spatstat.explore NAMESPACE file

## ................  Import packages ..................

import(stats,graphics,grDevices,utils,methods)
import(spatstat.utils,spatstat.data,
       spatstat.univar,spatstat.sparse,
       spatstat.geom,spatstat.random)
import(goftest)
import(Matrix)

importFrom(abind,abind)
importFrom(nlme, collapse)

## ................ Load dynamic library ..............
##      (native routines are registered in init.c)
##      (entry points are symbols with prefix "SE_")

useDynLib(spatstat.explore, .registration=TRUE, .fixes="SE_")

## //////////  DO NOT EDIT THE FOLLOWING  ///////////////
## ////////  it is generated automatically  /////////////





# .................................................. 
# Automatically-generated list of documented objects 
# .................................................. 
export("$<-.fv")
export("adaptive.density")
export("adjust.ratfv")
export("allstats")
export("alltypes")
export("ang2rad")
export("apply.ssf")
export("as.data.frame.bw.optim")
export("as.data.frame.envelope")
export("as.data.frame.fv")
export("as.function.fv")
export("as.function.rhohat")
export("as.function.ssf")
export("as.fv")
export("as.fv.bw.optim")
export("as.fv.data.frame")
export("as.fv.density")
export("as.fv.fasp")
export("as.fv.fv")
export("as.fv.matrix")
export("as.im.scan.test")
export("as.im.ssf")
export("as.owin.quadrattest")
export("as.ppp.ssf")
export("as.roc.data.frame")
export("assemble.plot.objects")
export("as.tess.quadrattest")
export("auc")
export("auc.bermantest")
export("auc.cdftest")
export("aucData")
export("auc.im")
export("auc.ppp")
export("auc.roc")
export("auc.spatialCDFframe")
export("bandwidth.is.infinite")
export("BartCalc")
export("berman.test")
export("bermantestCalc")
export("bermantestEngine")
export("berman.test.ppp")
export("bias.envelope")
export("bind.fv")
export("bind.ratfv")
export("bits.envelope")
export("bits.test")
export("blur")
export("blurHeat")
export("blurHeat.im")
export("boyce")
export("bw.abram.ppp")
export("bw.bdh")
export("bw.CvL")
export("bw.CvL.adaptive")
export("bw.CvLHeat")
export("bw.diggle")
export("bw.fiksel")
export("bw.frac")
export("bw.optim")
export("bw.pcf")
export("bw.pcfinhom")
export("bw.ppl")
export("bw.pplHeat")
export("bw.relrisk")
export("bw.relriskHeatppp")
export("bw.relrisk.ppp")
export("bw.scott")
export("bw.scott.iso")
export("bw.smoothppp")
export("bw.stoyan")
export("calc.DR")
export("calc.NNIR")
export("calc.SAVE")
export("calc.SIR")
export("calc.TSE")
export("cbind.fv")
export("cdf.test")
export("cdf.test.ppp")
export("censtimeCDFest")
export("check.testfun")
export("circdensity")
export("circticks")
export("clarkevans")
export("clarkevansCalc")
export("clarkevans.test")
export("clusterset")
export("collapse.anylist")
export("collapse.fv")
export("compatible.fasp")
export("compatible.fv")
export("compatible.rat")
export("compileCDF")
export("compileK")
export("compilepcf")
export("Complex.fasp")
export("Complex.fv")
export("conform.ratfv")
export("contour.ssf")
export("cor.im")
export("cov.im")
export("coxmap")
export("coxmap.ppp")
export("CressieReadName")
export("CressieReadStatistic")
export("CressieReadSymbol")
export("cutoff2Dkernel")
export("CVforPCF")
export("dclf.progress")
export("dclf.sigtrace")
export("dclf.test")
export("DeltaMethodVarOfRatio")
export("densityAdaptiveKernel.ppp")
export("densityAdaptiveKernel.ppplist")
export("densityAdaptiveKernel.splitppp")
export("densitycrossEngine")
export("densityfun")
export("densityfun.ppp")
export("densityHeat")
export("densityHeat.ppp")
export("densitypointsEngine")
export("density.ppp")
export("density.ppplist")
export("density.psp")
export("density.splitppp")
export("densityVoronoi")
export("densityVoronoi.ppp")
export("deriv.fv")
export("Deviation")
export("dg.envelope")
export("dg.progress")
export("dg.sigtrace")
export("dg.test")
export("digestCovariates")
export("digital.volume")
export("dim.fasp")               
export("dimhat")
export("dimnames.fasp")               
export("dimnames<-.fasp")
export("distcdf")
export("distributecbind")
export("domain.quadrattest")
export("edge.Ripley")
export("edge.Trans")
export("Emark")
export("ensure.listarg")
export("envelope")
export("envelopeArray")
export("envelopeEngine")
export("envelope.envelope")
export("envelope.hasenvelope")
export("envelope.matrix")
export("envelope.pp3")
export("envelope.ppp")
export("envelopeProgressData")
export("envelopeTest")
export("evalCovar")
export("eval.fasp")
export("eval.fv")
export("evaluate2Dkernel")
export("evaluateCovariate")
export("evaluateCovariateAtPixels")
export("evaluateCovariateAtPoints")
export("exactppm")
export("ExpSmoothLog")
export("extractAtomicQtests")
export("f3Cengine")
export("f3engine")
export("F3est")
export("[.fasp")
export("fasp")
export("Fest")
export("Fhazard")
export("findbestlegendpos")
export("findcbind")
export("Finhom")
export("fitted.exactppm")
export("flatfname")
export("Fmulti.inhom")
export("FmultiInhom")
export("FormatFaspFormulae")
export("formula<-")
export("formula.fv")
export("formula<-.fv")
export("fryplot")
export("frypoints")
export("[.fv")
export("[<-.fv")
export("fv")
export("fvexprmap")
export("fvlabelmap")
export("fvlabels")
export("fvlabels<-")
export("fvlegend")
export("fvnames")
export("fvnames<-")
export("g3Cengine")
export("g3engine")
export("G3est")
export("Gcross")
export("Gcross.inhom")
export("Gdot")
export("Gdot.inhom")
export("Gest")
export("getSumFun")
export("Gfox")
export("Ginhom")
export("Gmulti")
export("Gmulti.inhom")
export("GmultiInhom")
export("good.correction.K")
export("harmonise.fv")
export("harmonize.fv")
export("hasenvelope")
export("Hest")
export("hopskel")
export("hopskel.test")
export("hotbox")
export("idw")
export("Iest")
export("image.ssf")
export("implemented.for.K")
export("increment.fv")
export("integral.fv")
export("integral.ssf")
export("is.atomicQtest")
export("ISB.envelope")
export("ISE.envelope")
export("is.poisson.exactppm")
export("is.scov")
export("is.stationary.exactppm")
export("IV.envelope")
export("Jcross")
export("Jcross.inhom")
export("Jdot")
export("Jdot.inhom")
export("Jest")
export("Jfox")
export("Jinhom")
export("Jmulti")
export("Jmulti.inhom")
export("k3engine")
export("K3est")
export("Kborder.engine")
export("Kcross")
export("Kcross.inhom")
export("Kdot")
export("Kdot.inhom")
export("Kest")
export("Kest.fft")
export("Kinhom")
export("Kmark")
export("Kmeasure")
export("Kmulti")
export("Kmulti.inhom")
export("Knone.engine")
export("Kount")
export("Krect.engine")
export("Kscaled")
export("Ksector")
export("Kwtsum")               
export("laslett")
export("Lcross")
export("Lcross.inhom")
export("Ldot")
export("Ldot.inhom")
export("Lest")
export("Linhom")
export("localK")
export("localKcross")
export("localKcross.inhom")
export("localKdot")
export("localKengine")
export("localKinhom")
export("localKmultiEngine")
export("localL")
export("localLcross")
export("localLcross.inhom")
export("localLdot")
export("localLinhom")
export("localpcf")
export("localpcfengine")
export("localpcfinhom")
export("[.localpcfmatrix")
export("localpcfmatrix")
export("lohboot")
export("lookup2DkernelInfo")
export("Lscaled")
export("mad.progress")
export("mad.sigtrace")
export("mad.test")
export("makebreaksinside")
export("makefvlabel")
export("makeRocDesc")
export("makeRocFlabel")
export("makeRocTag")
export("make.simulrecipe")
export("make.simulrecipe.clusterprocess")
export("make.simulrecipe.pp3")
export("make.simulrecipe.ppp")
export("markconnect")
export("markcorr")
export("markcorrint")
export("markcrosscorr")
export("markequal")
export("markmarkscatter")
export("markmean")
export("marks.ssf")
export("marks<-.ssf")
export("marktable")
export("markvar")
export("markvario")
export("maskLaslett")
export("match2DkernelName")
export("Math.fasp")
export("Math.fv")
export("max.ssf")
export("mctest.progress")
export("mctest.sigtrace")
export("mctestSigtraceEngine")
export("meanlistfv")
export("min.ssf")
export("miplot")
export("MISE.envelope")
export("monotonicRhoFun")
export("monotonicRhoFunCalc")
export("names<-.fv")
export("nearest.neighbour")
export("needROC")
export("nnclean")
export("nncleanEngine")
export("nnclean.pp3")
export("nnclean.ppp")
export("nncorr")
export("nncount")
export("nndensity")
export("nndensity.ppp")
export("nnequal")
export("nnmean")
export("nnorient")
export("nnvario")
export("Ops.fasp")
export("Ops.fv")
export("pairMean")
export("pairorient")
export("pairs.im")
export("pairs.listof")
export("pairs.solist")
export("panel.contour")
export("panel.histogram")
export("panel.image")
export("pcf")
export("pcf3engine")
export("pcf3est")
export("pcfcross")
export("pcfcross.inhom")
export("pcfdot")
export("pcfdot.inhom")
export("pcf.fasp")
export("pcf.fv")
export("pcfinhom")
export("pcfmulti")
export("pcfmulti.inhom")
export("pcf.NAobject")
export("pcf.ppp")
export("plot.bermantest")
export("plot.bw.frac")
export("plot.bw.optim")
export("plot.cdftest")
export("plot.envelope")
export("plot.fasp")
export("plot.fv")
export("plot.laslett")
export("plot.localpcfmatrix")
export("plot.plotpairsim")
export("plot.quadrattest")
export("plot.rho2hat")
export("plot.rhohat")
export("plot.roc")
export("plot.scan.test")
export("plot.spatialcdf")
export("plot.ssf")
export("plot.studpermutest")
export("plot.tolcon")
export("plot.tolconlist")
export("polyLaslett")
export("pool")
export("pool.anylist")
export("pool.envelope")
export("pool.fasp")
export("pool.fv")
export("pool.quadrattest")
export("pool.rat")
export("posify")
export("PPversion")
export("predict.exactppm")
export("predict.rho2hat")
export("predict.rhohat")
export("prefixfv")
export("prettyweird")
export("print.bw.frac")
export("print.bw.optim")
export("print.densityfun")
export("print.envelope")
export("print.exactppm")
export("print.fasp")       
export("print.fv")       
export("print.fvfun")       
export("print.hasenvelope")       
export("print.laslett")
export("print.localpcfmatrix")
export("print.plotpairsim")
export("print.quadrattest")
export("print.rat")
export("print.rho2hat")
export("print.rhohat")
export("print.Smoothfun")       
export("print.ssf")
export("print.summary.bw.optim")
export("print.summary.ssf")
export("print.tolcon")
export("ptwise.envelope")
export("QQversion")
export("quadrat.test")
export("quadrat.testEngine")
export("quadrat.test.ppp")
export("quadrat.test.quadratcount")
export("quadrat.test.splitppp")
export("radcumint")
export("range.ssf")
export("[.rat")
export("rat")
export("ratfv")
export("rebadge.as.crossfun")
export("rebadge.as.dotfun")
export("rebadge.fv")
export("rebadgeLabels")
export("reconcile.fv")
export("rectcontact")
export("RelevantDeviation")
export("reload.or.compute")
export("relrisk")
export("relriskHeat")
export("relriskHeat.ppp")
export("relrisk.ppp")
export("rename.fv")
export("resolve.2D.kernel")
export("resolve.angular.unit")        
export("resolveEinfo")
export("resolve.foxall.window")
export("resolve.hist.args")
export("resolve.lambda")
export("resolve.lambdacross")
export("resolve.lambdacross.ppp")
export("resolve.lambda.ppp")
export("resolveNullModel")
export("resolve.pcf.bandwidth")
export("resolve.reciplambda")
export("resolve.reciplambda.ppp")
export("resolve.rose.args")
export("rho2hat")
export("rhohat")
export("rhohatCalc")
export("rhohatEngine")
export("rhohat.ppp")
export("rhohat.quad")
export("rmax.Rigid")
export("rmax.Ripley")
export("rmax.rule")
export("rmax.Trans")
export("RMSE.envelope")
export("roc")
export("roc.bermantest")
export("roc.cdftest")
export("rocData")
export("rocDummy")
export("rocEngine")
export("roc.function")
export("roc.im")
export("rocIm")
export("rocModel") 
export("roc.ppp")
export("roc.rhohat")
export("rocSmoothCalc")
export("roc.spatialCDFframe")
export("rose")
export("roseContinuous")
export("rose.default")
export("rose.density")
export("rose.fv")
export("rose.histogram")
export("rotmean")
export("scanBinomLRTS")
export("scanLRTS")
export("scanmeasure")
export("scanmeasure.im")
export("scanmeasure.ppp")
export("scanPoisLRTS")
export("scan.test")
export("sdr")
export("sdr.ppp")
export("sdrPredict")
export("second.moment.calc")
export("second.moment.engine")
export("segregation.test")
export("segregation.test.ppp")
export("sewpcf")
export("sewsmod")
export("sharpen")
export("sharpen.ppp")
export("shift.quadrattest")              
export("shift.tolcon")              
export("shift.tolconlist")              
export("simulate.rhohat")
export("simulrecipe")              
export("Smooth")
export("smoothcrossEngine")              
export("Smoothfun")
export("Smoothfun.ppp")
export("Smooth.fv")
export("SmoothHeat")
export("SmoothHeat.im")
export("SmoothHeat.ppp")
export("Smooth.im")
export("smoothpointsEngine")
export("Smooth.ppp")
export("Smooth.solist")
export("Smooth.ssf")
export("spatcov")
export("spatialcdf")
export("spatialCDFframe")
export("spatialCDFtest")
export("spatialCDFtestCalc")
export("spatialCovariateEvidence")
export("spatialCovariateEvidence.exactppm")
export("spatialCovariateEvidence.ppp")
export("SpatialMedian")
export("SpatialMedian.ppp")
export("SpatialQuantile")
export("SpatialQuantile.ppp")
export("sphere.volume")
export("[.ssf")
export("ssf")
export("StieltjesCalc.fv")
export("stienen")
export("stienenSet")
export("studpermu.test")
export("subspaceDistance")
export("summary.bw.optim")
export("summary.envelope")
export("Summary.fasp")
export("Summary.fv")
export("summary.ssf")
export("thresholdCI")
export("thresholdSelect")
export("tolcon")
export("transect.im")
export("Tstat")
export("tweak.fv.entry")
export("tweak.ratfv.entry")
export("twostage.envelope")
export("twostage.test")
export("unmark.ssf")
export("updateData")
export("updateData.default")
export("validate2Dkernel")
export("validate.weights")        
export("vanilla.fv")
export("varblock")
export("VarOfWtdMean")
export("Vmark")
export("weightedclosepairs")
export("which.max.im")
export("Window.quadrattest")
export("with.fv")
export("with.ssf")
export("X2testEngine")
export("youden")

# .......  Special cases ........... 
S3method("Complex", "fasp")
S3method("Complex", "fv")
S3method("Math", "fasp")
S3method("Math", "fv")
S3method("Ops", "fasp")
S3method("Ops", "fv")
S3method("Summary", "fasp")
S3method("Summary", "fv")
# .......  End of special cases  ... 

# ......................................... 
# Automatically generated list of S3 methods
# ......................................... 
S3method("as.data.frame", "bw.optim")
S3method("as.data.frame", "envelope")
S3method("as.data.frame", "fv")
  S3method("as.function", "fv")
  S3method("as.function", "rhohat")
  S3method("as.function", "ssf")
  S3method("as.fv", "bw.optim")
  S3method("as.fv", "data.frame")
  S3method("as.fv", "density")
  S3method("as.fv", "fasp")
  S3method("as.fv", "fv")
  S3method("as.fv", "matrix")
 S3method("as.im", "scan.test")
  S3method("as.im", "ssf")
 S3method("as.owin", "quadrattest")
  S3method("as.ppp", "ssf")
 S3method("as.tess", "quadrattest")
S3method("auc", "bermantest")
S3method("auc", "cdftest")
S3method("auc", "im")
S3method("auc", "ppp")
S3method("auc", "roc")
S3method("auc", "spatialCDFframe")
S3method("berman.test", "ppp")
S3method("blurHeat", "im")
S3method("bw.abram", "ppp")
   S3method("bw.relrisk", "ppp")
S3method("cbind", "fv")
S3method("cdf.test", "ppp")
S3method("collapse", "anylist")
S3method("collapse", "fv")
  S3method("compatible", "fasp")
  S3method("compatible", "fv")
S3method("compatible", "rat")
S3method("contour", "ssf")
S3method("coxmap", "ppp")
S3method("densityAdaptiveKernel", "ppp")
S3method("densityAdaptiveKernel", "ppplist")
S3method("densityAdaptiveKernel", "splitppp")
S3method("densityfun", "ppp")
S3method("densityHeat", "ppp")
  S3method("density", "ppp")
  S3method("density", "ppplist")
  S3method("density", "psp")
  S3method("density", "splitppp")
S3method("densityVoronoi", "ppp")
S3method("deriv", "fv")
S3method("dim", "fasp")
S3method("dimnames", "fasp")
 S3method("domain", "quadrattest")
S3method("envelope", "envelope")
S3method("envelope", "hasenvelope")
S3method("envelope", "matrix")
  S3method("envelope", "pp3")
  S3method("envelope", "ppp")
  S3method("[", "fasp")
S3method("fitted", "exactppm")
S3method("formula", "fv")
  S3method("[", "fv")
S3method("harmonise", "fv")
S3method("harmonize", "fv")
S3method("image", "ssf")
S3method("integral", "fv")
  S3method("integral", "ssf")
S3method("is.poisson", "exactppm")
S3method("is.stationary", "exactppm")
S3method("[", "localpcfmatrix")
S3method("make.simulrecipe", "clusterprocess")
S3method("make.simulrecipe", "pp3")
S3method("make.simulrecipe", "ppp")
  S3method("marks", "ssf")
  S3method("max", "ssf")
  S3method("min", "ssf")
  S3method("nnclean", "pp3")
  S3method("nnclean", "ppp")
S3method("nndensity", "ppp")
S3method("pairs", "im")
S3method("pairs", "listof")
S3method("pairs", "solist")
 S3method("pcf", "fasp")
  S3method("pcf", "fv")
S3method("pcf", "NAobject")
  S3method("pcf", "ppp")
S3method("plot", "bermantest")
S3method("plot", "bw.frac")
S3method("plot", "bw.optim")
S3method("plot", "cdftest")
 S3method("plot", "envelope")
   S3method("plot", "fasp")
 S3method("plot", "fv")
S3method("plot", "laslett")
S3method("plot", "localpcfmatrix")
S3method("plot", "plotpairsim")
 S3method("plot", "quadrattest")
S3method("plot", "rho2hat")
S3method("plot", "rhohat")
S3method("plot", "roc")
 S3method("plot", "scan.test")
S3method("plot", "spatialcdf")
S3method("plot", "ssf")
  S3method("plot", "studpermutest")
S3method("plot", "tolcon")
S3method("plot", "tolconlist")
S3method("pool", "anylist")
S3method("pool", "envelope")
S3method("pool", "fasp")
S3method("pool", "fv")
S3method("pool", "quadrattest")
S3method("pool", "rat")
S3method("predict", "exactppm")
S3method("predict", "rho2hat")
S3method("predict", "rhohat")
S3method("print", "bw.frac")
S3method("print", "bw.optim")
S3method("print", "densityfun")
S3method("print", "envelope")
S3method("print", "exactppm")
S3method("print", "fasp")
S3method("print", "fv")
S3method("print", "fvfun")
S3method("print", "hasenvelope")
S3method("print", "laslett")
S3method("print", "localpcfmatrix")
S3method("print", "plotpairsim")
S3method("print", "quadrattest")
S3method("print", "rat")
S3method("print", "rho2hat")
S3method("print", "rhohat")
S3method("print", "Smoothfun")
  S3method("print", "ssf")
S3method("print", "summary.bw.optim")
S3method("print", "summary.ssf")
S3method("print", "tolcon")
S3method("quadrat.test", "ppp")
S3method("quadrat.test", "quadratcount")
S3method("quadrat.test", "splitppp")
  S3method("range", "ssf")
S3method("[", "rat")
S3method("relriskHeat", "ppp")
S3method("relrisk", "ppp")
S3method("resolve.lambdacross", "ppp")
S3method("resolve.lambda", "ppp")
S3method("resolve.reciplambda", "ppp")
S3method("rhohat", "ppp")
S3method("rhohat", "quad")
S3method("roc", "bermantest")
S3method("roc", "cdftest")
S3method("roc", "function")
S3method("roc", "im")
S3method("roc", "ppp")
S3method("roc", "rhohat")
S3method("roc", "spatialCDFframe")
S3method("rose", "default")
S3method("rose", "density")
S3method("rose", "fv")
S3method("rose", "histogram")
S3method("scanmeasure", "im")
S3method("scanmeasure", "ppp")
S3method("sdr", "ppp")
S3method("segregation.test", "ppp")
S3method("sharpen", "ppp")
S3method("shift", "quadrattest")
S3method("shift", "tolcon")
S3method("shift", "tolconlist")
S3method("simulate", "rhohat")
S3method("Smoothfun", "ppp")
S3method("Smooth", "fv")
S3method("SmoothHeat", "im")
S3method("SmoothHeat", "ppp")
S3method("Smooth", "im")
S3method("Smooth", "ppp")
S3method("Smooth", "solist")
 S3method("Smooth", "ssf")
S3method("spatialCovariateEvidence", "exactppm")
S3method("spatialCovariateEvidence", "ppp")
 S3method("SpatialMedian", "ppp")
 S3method("SpatialQuantile", "ppp")
 S3method("[", "ssf")
S3method("StieltjesCalc", "fv")
S3method("summary", "bw.optim")
S3method("summary", "envelope")
  S3method("summary", "ssf")
  S3method("unmark", "ssf")
S3method("updateData", "default")
 S3method("Window", "quadrattest")
S3method("with", "fv")
  S3method("with", "ssf")
# ......................................... 
#      Assignment methods                   
# ......................................... 
  S3method("$<-", "fv")
S3method("dimnames<-", "fasp")
S3method("formula<-", "fv")
  S3method("[<-", "fv")
  S3method("marks<-", "ssf")
S3method("names<-", "fv")
# ......................................... 
#      End of methods                       
# ......................................... 
